#include <Arduino.h>
#include <Motor/Motor.h>
#include <Motor/MotorControl.h>
#include <constants.h>

using namespace constants;

Motor faceFront(PIN_ENABLED_FF);
Motor faceRight(PIN_ENABLED_FR);
Motor faceBack(PIN_ENABLED_FB);
Motor faceLeft(PIN_ENABLED_FL);
Motor faceUp(PIN_ENABLED_FU);
Motor faceDown(PIN_ENABLED_FD);

MotorControl::MotorControl(){    
    pinMode(PIN_DIRECTION, OUTPUT);
    pinMode(PIN_STEPS, OUTPUT);
}

void MotorControl::executeMovement(String sequence){
    //Receives a string with the movements
    //Pattern: U L' F' B' D F' L U F' B' L B' U D2 F2 L2 U R2 F2 R2 U2 D
    //U: Up
    //L: Left
    //F: Front
    //R: Right
    //D: Down
    //B: Back
    //': Anticlockwise
    //2: Double turn
    //Serial.println(sequence);
    Movement currentMove;
    for (int i = 0; i < sequence.length(); i++){
        //CHECK HERE WHICH COMMAND IS ARRIVING!!!
        if (
            (sequence[i] == *"F") ||
            (sequence[i] == *"R") ||
            (sequence[i] == *"B") ||
            (sequence[i] == *"L") ||
            (sequence[i] == *"U") ||
            (sequence[i] == *"D")
        ){
            currentMove.doubleIt = false;
            currentMove.rotation = clockwise;
            switch (sequence[i]){
                case 'F':
                    currentMove.motor = &faceFront;
                    break;
                case 'R':
                    currentMove.motor = &faceRight;
                    break;
                case 'B':
                    currentMove.motor = &faceBack;
                    break;
                case 'L':
                    currentMove.motor = &faceLeft;
                    break;
                case 'U':
                    currentMove.motor = &faceUp;
                    break;
                case 'D':
                    currentMove.motor = &faceDown;
                    break;
            }
        } else
            if (sequence[i] == *"'"){
                currentMove.rotation = counterClockwise;
            } else
                if (sequence[i] == *"2"){
                    currentMove.doubleIt = true;
                }
        if ((sequence[i] == *" ")||(i == sequence.length() - 1)){
            //perform the movement
            
            //Reverses direction on front and rear motors
            // if (
            //     (currentMove.motor == &faceFront) ||
            //     (currentMove.motor == &faceBack) ||
            //     (currentMove.motor == &faceDown)
            // ){
            //     currentMove.rotation = (currentMove.rotation == counterClockwise) ? clockwise : counterClockwise;
            // }
            currentMove.motor->rotate(
                currentMove.rotation, 
                (currentMove.doubleIt) ? 2 : 1
            );
        }
    }                    
}