//pio run && pio run --target upload --upload-port /dev/ttyUSB0 && pio device monitor
#include <Arduino.h>
#include <Motor/Motor.h>
#include <Motor/MotorControl.h>
#include <constants.h>
#include <string>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>

using namespace constants;

ESP8266WebServer server(80);
 
const char* ssid = "x";
const char* password =  "x";

bool _processing;
MotorControl controller;

void setProcessing(bool value){    
    _processing = value;
    digitalWrite(PIN_ONBOARD_LED, 
        (_processing) ? HIGH : LOW
    );
}

void setupMotors(){
    // put your setup code here, to run once:
    Serial.begin(115200);//9600);   
    Serial.println("Started...");
    pinMode(PIN_ONBOARD_LED, OUTPUT);
    controller = MotorControl();
}

void processBySerialMonitor(){
     if (Serial.available()){
        //receives the movement sequence of the motors through the serial port
        if (!_processing){
            setProcessing(true);
            String movementSequence = Serial.readStringUntil('\r\n');
            Serial.println(movementSequence);
            controller.executeMovement(movementSequence);
            setProcessing(false);
        } else {
            Serial.println("Wait, processing...");
        }
    } 
}

void processByWebPage(){
    server.handleClient(); //Handling of incoming requests
}

void handlePost() { //Handler for the post path
    if (server.hasArg("plain")== false){ //Check if body received
        server.send(200, "text/plain", "Body not received");
        return;
    }

    //start solving
    //receives the movement sequence of the motors through the post request
    if (!_processing){
        setProcessing(true);
        String movementSequence = server.arg("plain");
        Serial.println("Received from post:");
        Serial.println(movementSequence);
        controller.executeMovement(movementSequence);
        setProcessing(false);
    } else {
        Serial.println("Wait, processing previous request...");
    }

    server.send(200, "text/plain", "Success");
    ////
}

String SendHTML(){
  String strVar="";
    strVar += "<!DOCTYPE html>";
    strVar += "<html lang=\"en\">";
    strVar += "";
    strVar += "  <head>";
    strVar += "    <title>Coobycs<\/title>";
    strVar += "    <script src=\"https:\/\/ajax.googleapis.com\/ajax\/libs\/jquery\/3.1.1\/jquery.min.js\"><\/script>";
    strVar += "    <script src=\"https:\/\/unpkg.com\/cube-solver@2.4.1\/dist\/bundle.js\"><\/script>";
    strVar += "    <script src=\"https:\/\/cdn.jsdelivr.net\/gh\/larspetrus\/Roofpig\/roofpig_and_three.min.js\"><\/script>";
    strVar += "";
    strVar += "    <script>";
    strVar += "      $(document).ready(function() {";
    strVar += "        const appDiv = document.getElementById('app');";
    strVar += "        cubeSolver.initialize('kociemba');";
    strVar += "        var scramble = '';";
    strVar += "";
    strVar += "        function scrambleCube() {";
    strVar += "          scramble = cubeSolver.scramble();";
    strVar += "          var scrambleSpan = document.getElementById('scrambleVal');";
    strVar += "          scrambleSpan.value = scramble;";
    strVar += "          return scramble;";
    strVar += "        }";
    strVar += "";
    strVar += "        function solveCube(scramble) {";
    strVar += "";
    strVar += "          solve = scramble == '' ? '' : cubeSolver.solve(scramble);";
    strVar += "          var solveSpan = document.getElementById('solveVal');";
    strVar += "          solveSpan.value = solve;";
    strVar += "        }";
    strVar += "        $('#solveValButtonRobot').click(function() {";
    strVar += "          var solveSend = document.getElementById('solveVal').value;";
    strVar += "          if (solveSend) {";
    strVar += "            $.ajax({";
    strVar += "              url: 'solve',";
    strVar += "              data: solveSend,";
    strVar += "              type: 'post',";
    strVar += "              contentType: \"text\/plain\",";
    strVar += "              error: function(XMLHttpRequest, textStatus, errorThrown) {";
    strVar += "                alert(textStatus);";
    strVar += "              },";
    strVar += "              success: function() {";
    strVar += "                alert(\"Sent to robot!\");";
    strVar += "              }";
    strVar += "            });";
    strVar += "          }";
    strVar += "        });";
    strVar += "        $(\"#scrambleBtn\").click(function() {";
    strVar += "          var result = scrambleCube();";
    strVar += "          solveCube('');";
    strVar += "          $(\".cube\").children().remove();";
    strVar += "          CubeAnimation.create_in_dom('.cube', \"alg=\" + result + \"|flags=FALSE\");";
    strVar += "        });";
    strVar += "";
    strVar += "        $(\"#solveBtn\").click(function() {";
    strVar += "          var scrambleToSolve = document.getElementById('scrambleVal').value;";
    strVar += "          solveCube(scrambleToSolve);";
    strVar += "          $(\".cube\").children().remove();";
    strVar += "          CubeAnimation.create_in_dom('.cube', \"\");";
    strVar += "        })";
    strVar += "      });";
    strVar += "";
    strVar += "    <\/script>";
    strVar += "    <style>";
    strVar += "      body {";
    strVar += "        font-size: 10px;";
    strVar += "      }";
    strVar += "";
    strVar += "      div.cube {";
    strVar += "        width: 150px;";
    strVar += "        height: 193px;";
    strVar += "        margin: 5px;";
    strVar += "        float: left;";
    strVar += "      }";
    strVar += "";
    strVar += "    <\/style>";
    strVar += "  <\/head>";
    strVar += "";
    strVar += "  <body>";
    strVar += "";
    strVar += "    <div id=\"app\">";
    strVar += "      <h1>Coobycs Robot solver<\/h1>";
    strVar += "    <\/div>";
    strVar += "    <div class=\"cube\">";
    strVar += "      <div class=\"roofpig\"><\/div>";
    strVar += "    <\/div>";
    strVar += "    <button id=\"scrambleBtn\">Scramble<\/button>";
    strVar += "    <br>";
    strVar += "    <input id=\"scrambleVal\">";
    strVar += "    <br>";
    strVar += "    <button id=\"solveBtn\">Solve<\/button>";
    strVar += "    <br>";
    strVar += "    <input id=\"solveVal\">";
    strVar += "    <br>";
    strVar += "    <button id=\"solveValButtonRobot\">Send to robot<\/button>";
    strVar += "  <\/body>";
    strVar += "";
    strVar += "<\/html>";
    strVar += "";

  return strVar;
}

void handle_OnConnect() {
  server.send(200, "text/html", SendHTML()); 
}

void setupWebPage(){
    WiFi.begin(ssid, password);  //Connect to the WiFi network
 
    while (WiFi.status() != WL_CONNECTED) {  //Wait for connection
        delay(500);
        Serial.println("Waiting to connect to wifi...");
    }
 
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());  //Print the local IP
 
    server.on("/", handle_OnConnect);
    server.on("/solve", handlePost); //Associate the handler function to the path
 
    server.begin(); //Start the server
    Serial.println("Server listening");
}

void setup() {
    setupMotors();
    setupWebPage();
}

void loop() {
        //processBySerialMonitor();
        processByWebPage();
}   